var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: !0, configurable: !0, writable: !0, value }) : obj[key] = value;
var __name = (target, value) => __defProp(target, "name", { value, configurable: !0 });
var __publicField = (obj, key, value) => (__defNormalProp(obj, typeof key != "symbol" ? key + "" : key, value), value);
import { AxiosError } from "axios";
import { ref, computed, inject, defineComponent, watchEffect, openBlock, createElementBlock, renderSlot } from "vue";
const _ApiError = class _ApiError extends AxiosError {
  constructor(message, code, config, request, response) {
    super(message, code, config, request, response);
    __publicField(this, "response");
    __publicField(this, "enableAutoError", !0);
    this.response = response;
  }
  getMessage() {
    var _a;
    return (_a = this.response) == null ? void 0 : _a.data.message;
  }
  showError() {
    if (this.getMessage()) {
      alert(this.getMessage());
      return;
    }
    alert(this.message);
  }
};
__name(_ApiError, "ApiError");
let ApiError = _ApiError;
const _ValidateError = class _ValidateError extends ApiError {
  constructor(message, code, config, request, response) {
    super(message, code, config, request, response);
    __publicField(this, "enableAutoError", !1);
    __publicField(this, "errors");
    __publicField(this, "response");
    this.errors = response == null ? void 0 : response.data.errors, this.response = response;
  }
  hasError(field = "") {
    return this.response ? field ? !!this.getErrorMessage(field) : !!this.response.data.message : !1;
  }
  getErrorMessage(field = "") {
    var _a;
    if (!this.errors)
      return null;
    if (!field)
      return (_a = this.response) == null ? void 0 : _a.data.message;
    if (this.errors[field])
      return this.errors[field].join(`
`) || null;
  }
  getMessages() {
    var _a;
    return this.hasError() ? Object.keys(this.errors ?? {}).map((x) => this.errors[x]).join(`
`) : (_a = this.response) == null ? void 0 : _a.data.message;
  }
};
__name(_ValidateError, "ValidateError");
let ValidateError = _ValidateError;
const _GenerateErrorService = class _GenerateErrorService {
  constructor(error) {
    __publicField(this, "error", new AxiosError());
    __publicField(this, "response", null);
    error && (this.error = error, this.response = error.response);
  }
  createError(errors) {
    const e = errors.filter((x) => x.codeCondition(this.response.status)).map(
      (x) => new x.error(
        this.error.message,
        this.error.code,
        this.error.config,
        this.error.request,
        this.error.response
      )
    );
    return e.length ? e[0] : this.error;
  }
};
__name(_GenerateErrorService, "GenerateErrorService");
let GenerateErrorService = _GenerateErrorService;
const ValidateKey = Symbol("ValidateContext");
function provideValidateContext() {
  const validError = ref(), setValidError = /* @__PURE__ */ __name((error) => {
    validError.value = error;
  }, "setValidError"), errorMessage = computed(() => ""), validState = computed(() => !1);
  return {
    validError,
    setValidError,
    errorMessage,
    validState
  };
}
__name(provideValidateContext, "provideValidateContext");
function useValidate(field) {
  const context = inject(ValidateKey);
  if (!context)
    throw new Error();
  const errorMessage = computed(() => {
    var _a, _b;
    return (_a = context.validError.value) != null && _a.hasError(field) ? (_b = context.validError.value) == null ? void 0 : _b.getErrorMessage(field) : "";
  }), validState = computed(() => {
    var _a;
    return (_a = context.validError.value) != null && _a.hasError(field) ? !1 : null;
  });
  return {
    ...context,
    errorMessage,
    validState,
    clearValidate: /* @__PURE__ */ __name(() => {
      context.setValidError(new ValidateError());
    }, "clearValidate")
  };
}
__name(useValidate, "useValidate");
const _DefaultError = class _DefaultError extends ApiError {
  constructor(message, code, config, request, response) {
    super(message, code, config, request, response);
    __publicField(this, "enableAutoError", !0);
  }
};
__name(_DefaultError, "DefaultError");
let DefaultError = _DefaultError;
const ValidatePlugin = {
  install(app, options) {
    options != null && options.manual || (app.provide(ValidateKey, provideValidateContext()), this.attachAxiosInterceptor(options));
  },
  attachAxiosInterceptor(options) {
    var _a;
    (_a = options == null ? void 0 : options.axios) == null || _a.interceptors.response.use(
      (response) => Promise.resolve(response),
      (error) => {
        var _a2;
        const e = new GenerateErrorService(error), errors = /* @__PURE__ */ new Set();
        return errors.add(
          { codeCondition: (n) => n == 422, error: ValidateError }
        ), (_a2 = options == null ? void 0 : options.registerError) == null || _a2.forEach((x) => {
          errors.add(x);
        }), errors.add(
          { codeCondition: (n) => n != 422, error: DefaultError }
        ), Promise.reject(
          e.createError([...errors.values()])
        );
      }
    );
  }
}, _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "ValidateWrap",
  props: {
    attributeField: {
      type: String,
      default: "data"
    }
  },
  setup(__props) {
    const { validError } = useValidate(), slotContainer = ref(null), props = __props, clearErrorMessages = /* @__PURE__ */ __name(() => {
      var _a, _b;
      (_a = slotContainer.value) == null || _a.querySelectorAll(".invalid-feedback").forEach((x) => x.remove()), (_b = slotContainer.value) == null || _b.querySelectorAll(".is-invalid").forEach((x) => x.classList.remove("is-invalid"));
    }, "clearErrorMessages"), findInputElement = /* @__PURE__ */ __name((dataAttribute) => {
      var _a;
      return (_a = slotContainer.value) == null ? void 0 : _a.querySelectorAll(
        `[${props.attributeField}="${dataAttribute}"]`
      );
    }, "findInputElement"), showErrorMessage = /* @__PURE__ */ __name((element, message) => {
      element.classList.add("is-invalid");
      const errorDiv = document.createElement("div");
      errorDiv.classList.add("d-block", "invalid-feedback"), errorDiv.style.whiteSpace = "pre-wrap", errorDiv.textContent = message, element.after(errorDiv);
    }, "showErrorMessage");
    return watchEffect(() => {
      var _a;
      if (!((_a = validError.value) != null && _a.errors)) {
        clearErrorMessages();
        return;
      }
      clearErrorMessages();
      const { errors } = validError.value;
      Object.keys(errors).forEach((x) => {
        const errorMessage = errors[x].join(""), inputElements = findInputElement(x);
        inputElements != null && inputElements.length && (inputElements == null || inputElements.forEach((inputElement) => {
          showErrorMessage(inputElement, errorMessage);
        }));
      });
    }), (_ctx, _cache) => (openBlock(), createElementBlock("div", {
      ref_key: "slotContainer",
      ref: slotContainer
    }, [
      renderSlot(_ctx.$slots, "default")
    ], 512));
  }
});
export {
  ApiError,
  GenerateErrorService,
  ValidateError,
  ValidatePlugin,
  _sfc_main as ValidateWrap,
  provideValidateContext,
  useValidate
};
